<?php
/**
 * Handle license fields
 *
 * @package         EDD\S214\License
 * @since           1.0.0
 */


// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
}


/**
 * Add the callback for our custom license field
 *
 * @since       1.0.0
 * @param       array $args The args passed by the settings
 * @return      void
 */
if ( ! function_exists( 'edd_s214_license_key_callback' ) ) {
	function edd_s214_license_key_callback( $args ) {
		$messages = array();
		$license  = get_option( $args['id'] . '_status', false );

		if( edd_get_option( $args['id'], false ) ) {
			$value = edd_get_option( $args['id'], false );
		} else {
			$value = isset( $args['std'] ) ? $args['std'] : '';
		}

		if( ! empty( $license ) && is_object( $license ) ) {
			if( $license->success === false ) {
				switch( $license->error ) {
					case 'expired':
						$class = 'expired';
						$messages[] = sprintf(
							__( 'Your license key expired on %s. Please <a href="%s" target="_blank">renew your license key</a>.', 'section214-license' ),
							date_i18n( get_option( 'date_format' ), strtotime( $license->expires, current_time( 'timestamp' ) ) ),
							'https://section214.com/checkout/?edd_license_key=' . $value . '&utm_campaign=admin&utm_source=licenses&utm_medium=expired'
						);
						$license_status = 'license-' . $class . '-notice';
						break;
					case 'revoked':
						$class = 'error';
						$messages[] = sprintf(
							__( 'Your license key has been disabled. Please <a href="%s" target="_blank">contact support</a> for more information.', 'section214-license' ),
							'https://section214.com/contact?utm_campaign=admin&utm_source=licenses&utm_medium=revoked'
						);
						$license_status = 'license-' . $class . '-notice';
						break;
					case 'missing':
						$class = 'error';
						$messages[] = sprintf(
							__( 'Invalid license. Please <a href="%s" target="_blank">visit your account page</a> and verify it.', 'section214-license' ),
							'https://section214.com/your-account?utm_campaign=admin&utm_source=licenses&utm_medium=missing'
						);
						$license_status = 'license-' . $class . '-notice';
						break;
					case 'invalid':
					case 'site_inactive':
						$class = 'error';
						$messages[] = sprintf(
							__( 'Your %s is not active for this URL. Please <a href="%s" target="_blank">visit your account page</a> to manage your license key URLs.', 'section214-license' ),
							$args['name'],
							'https://section214.com/your-account?utm_campaign=admin&utm_source=licenses&utm_medium=invalid'
						);
						$license_status = 'license-' . $class . '-notice';
						break;
					case 'item_name_mismatch':
						$class = 'error';
						$messages[] = sprintf( __( 'This is not a %s.', 'section214-license' ), $args['name'] );
						$license_status = 'license-' . $class . '-notice';
						break;
					case 'no_activations_left':
						$class = 'error';
						$messages[] = sprintf( __( 'Your license key has reached its activation limit. <a href="%s">View possible upgrades</a> now.', 'section214-license' ), 'https://section214.com/your-account/' );
						$license_status = 'license-' . $class . '-notice';
						break;
					default:
						$messages[] = print_r( $license, true );
						break;
				}
			} else {
				switch( $license->license ) {
					case 'valid':
					default:
						$class      = 'valid';
						$now        = current_time( 'timestamp' );
						$expiration = strtotime( $license->expires, current_time( 'timestamp' ) );

						if( 'lifetime' === $license->expires ) {
							$messages[]     = __( 'License key never expires.', 'section214-license' );
							$license_status = 'license-lifetime-notice';
						} elseif( $expiration > $now && $expiration - $now < ( DAY_IN_SECONDS * 30 ) ) {
							$messages[] = sprintf(
								__( 'Your license key expires soon! It expires on %s. <a href="%s" target="_blank">Renew your license key</a>.', 'section214-license' ),
								date_i18n( get_option( 'date_format' ), strtotime( $license->expires, current_time( 'timestamp' ) ) ),
								'https://section214.com/checkout/?edd_license_key=' . $value . '&utm_campaign=admin&utm_source=licenses&utm_medium=renew'
							);
							$license_status = 'license-expires-soon-notice';
						} else {
							$messages[] = sprintf(
								__( 'Your license key expires on %s.', 'section214-license' ),
								date_i18n( get_option( 'date_format' ), strtotime( $license->expires, current_time( 'timestamp' ) ) )
							);
							$license_status = 'license-expiration-date-notice';
						}

						break;
				}
			}
		} else {
			$license_status = null;
		}

		$size = ( isset( $args['size'] ) && ! is_null( $args['size'] ) ) ? $args['size'] : 'regular';
		$html = '<input type="text" class="' . sanitize_html_class( $size ) . '-text" id="edd_settings[' . edd_sanitize_key( $args['id'] ) . ']" name="edd_settings[' . edd_sanitize_key( $args['id'] ) . ']" value="' . esc_attr( $value ) . '" />';

		if( ( is_object( $license ) && 'valid' == $license->license ) || 'valid' == $license ) {
			$html .= '<input type="submit" class="button-secondary" name="' . $args['id'] . '_deactivate" value="' . __( 'Deactivate License',  'section214-license' ) . '" />';
		} else {
			$html .= '<input type="hidden" name="' . $args['id'] . '_activate' . '" value="1" />';
		}

		$html .= '<label for="edd_settings[' . edd_sanitize_key( $args['id'] ) . ']"> '  . wp_kses_post( $args['desc'] ) . '</label>';

		if( ! empty( $messages ) ) {
			foreach( $messages as $message ) {
				$html .= '<div class="edd-license-data edd-license-' . $class . ' ' . $license_status . '">';
				$html .= '<p>' . $message . '</p>';
				$html .= '</div>';

			}
		}

		wp_nonce_field( edd_sanitize_key( $args['id'] ) . '-nonce', edd_sanitize_key( $args['id'] ) . '-nonce' );

		echo $html;
	}
}